import { Position, Range } from '@sourcegraph/extension-api-types';
import { Observable, Subscribable, SubscribableOrPromise, Subscription } from 'rxjs';
import { ErrorLike } from './errors';
import { DiffPart, PositionEvent } from './positions';
import { DOMFunctions, HoveredToken } from './token_position';
import { HoverAttachment, HoverOverlayProps, LineOrPositionOrRange, DocumentHighlight } from './types';
import { MaybeLoadingResult, LOADING } from './loading';
export { HoveredToken };
/**
 * @template C Extra context for the hovered token.
 * @template D The type of the hover content data.
 * @template A The type of an action.
 */
export interface HoverifierOptions<C extends object, D, A> {
    /**
     * Emit the HoverOverlay element on this after it was rerendered when its content changed and it needs to be repositioned.
     */
    hoverOverlayRerenders: Subscribable<{
        /**
         * The HoverOverlay element
         */
        hoverOverlayElement: HTMLElement;
        /**
         * The closest parent element that is `position: relative`
         */
        relativeElement: HTMLElement;
    }>;
    /**
     * Emit on this Observable when the close button in the HoverOverlay was clicked
     */
    closeButtonClicks: Subscribable<MouseEvent>;
    hoverOverlayElements: Subscribable<HTMLElement | null>;
    /**
     * Called to get the data to display in the hover.
     */
    getHover: HoverProvider<C, D>;
    /**
     * Called to get the set of ranges to highlight within the document.
     */
    getDocumentHighlights: DocumentHighlightProvider<C>;
    /**
     * Called to get the actions to display in the hover.
     */
    getActions: ActionsProvider<C, A>;
    /**
     * Whether or not hover tooltips can be pinned.
     */
    pinningEnabled: boolean;
    /**
     * Whether or not code views need to be tokenized. Defaults to true.
     */
    tokenize?: boolean;
    /**
     * The class name to apply to hovered tokens.
     */
    selectionHighlightClassName?: string;
    /**
     * The class name to apply to document highlight tokens.
     */
    documentHighlightClassName?: string;
}
/**
 * A Hoverifier is a function that hoverifies one code view element in the DOM.
 *
 * There can be multiple code views in the DOM, which will only show a single HoverOverlay if the same Hoverifier was used.
 *
 * @template C Extra context for the hovered token.
 * @template D The type of the hover content data.
 * @template A The type of an action.
 */
export interface Hoverifier<C extends object, D, A> {
    /**
     * The current Hover state. You can use this to read the initial state synchronously.
     */
    hoverState: Readonly<HoverState<C, D, A>>;
    /**
     * This Observable is to notify that the state that is used to render the HoverOverlay needs to be updated.
     */
    hoverStateUpdates: Observable<Readonly<HoverState<C, D, A>>>;
    /**
     * Hoverifies a code view.
     */
    hoverify(options: HoverifyOptions<C>): Subscription;
    unsubscribe(): void;
}
export interface PositionJump {
    /**
     * The position within the code view to jump to
     */
    position: LineOrPositionOrRange & {
        part?: DiffPart;
    };
    /**
     * The code view
     */
    codeView: HTMLElement;
    /**
     * The element to scroll if the position is out of view
     */
    scrollElement: HTMLElement;
}
/**
 * The possible directions to adjust a position in.
 */
export declare enum AdjustmentDirection {
    /** Adjusting the position from what is found on the page to what it would be in the actual file. */
    CodeViewToActual = 0,
    /** Adjusting the position from what is in the actual file to what would be found on the page. */
    ActualToCodeView = 1
}
/**
 * @template C Extra context for the hovered token.
 */
export interface AdjustPositionProps<C extends object> {
    /** The code view the token is in. */
    codeView: HTMLElement;
    /** The position the token is at. */
    position: HoveredToken & C;
    /** The direction the adjustment should go. */
    direction: AdjustmentDirection;
}
/**
 * Function to adjust positions coming into and out of hoverifier. It can be used to correct the position used in HoverProvider and
 * ActionsProvider requests and the position of the token to highlight in the code view. This is useful for code hosts that convert whitespace.
 *
 *
 * @template C Extra context for the hovered token.
 */
export declare type PositionAdjuster<C extends object> = (props: AdjustPositionProps<C>) => SubscribableOrPromise<Position>;
/**
 * HoverifyOptions that need to be included internally with every event
 *
 * @template C Extra context for the hovered token.
 */
export interface EventOptions<C extends object> {
    resolveContext: ContextResolver<C>;
    adjustPosition?: PositionAdjuster<C>;
    dom: DOMFunctions;
    codeViewId: symbol;
    /**
     * An array of elements used to hide the hover overlay if any of them
     * overlap with the hovered token. Overlapping is checked in reaction to scroll events.
     *
     * scrollBoundaries are typically elements with a lower z-index than the hover overlay
     * but a higher z-index than the code view, such as a sticky file header.
     */
    scrollBoundaries?: HTMLElement[];
    /**
     * Whether the specific code view should be tokenized. If defined, this is used over
     * the `tokenize` value passed to `createHoverifier`.
     *
     * Useful if some code views on a code host are already tokenized while other code views are not.
     */
    overrideTokenize?: boolean;
}
/**
 * @template C Extra context for the hovered token.
 */
export interface HoverifyOptions<C extends object> extends Pick<EventOptions<C>, Exclude<keyof EventOptions<C>, 'codeViewId'>> {
    positionEvents: Subscribable<PositionEvent>;
    /**
     * Emit on this Observable to trigger the overlay on a position in this code view.
     * This Observable is intended to be used to trigger a Hover after a URL change with a position.
     */
    positionJumps?: Subscribable<PositionJump>;
}
/**
 * Output that contains the information needed to render the HoverOverlay.
 *
 * @template C Extra context for the hovered token.
 * @template D The type of the hover content data.
 * @template A The type of an action.
 */
export interface HoverState<C extends object, D, A> {
    /**
     * The currently hovered and highlighted HTML element.
     */
    hoveredTokenElement?: HTMLElement;
    /**
     * Actions for the current token.
     */
    actionsOrError?: typeof LOADING | A[] | null | ErrorLike;
    /**
     * The props to pass to `HoverOverlay`, or `undefined` if it should not be rendered.
     */
    hoverOverlayProps?: Pick<HoverOverlayProps<C, D, A>, Exclude<keyof HoverOverlayProps<C, D, A>, 'actionComponent'>>;
    /**
     * The highlighted range, which is the range in the hover result or else the range of the hovered token.
     */
    highlightedRange?: Range;
    /**
     * The currently selected position, if any.
     * Can be a single line number or a line range.
     * Highlighted with a background color.
     */
    selectedPosition?: LineOrPositionOrRange;
}
/** The time in ms after which to show a loader if the result has not returned yet */
export declare const LOADER_DELAY = 600;
/** The time in ms after the mouse has stopped moving in which to show the tooltip */
export declare const TOOLTIP_DISPLAY_DELAY = 100;
/** The time in ms to debounce mouseover events. */
export declare const MOUSEOVER_DELAY = 50;
/**
 * Function that returns a Subscribable or PromiseLike of the hover result to be shown.
 * If a Subscribable is returned, it may emit more than once to update the content,
 * and must indicate when it starts and stopped loading new content.
 * It should emit a `null` result if the token has no hover content (e.g. whitespace, punctuation).
 *
 * @template C Extra context for the hovered token.
 * @template D The type of the hover content data.
 */
export declare type HoverProvider<C extends object, D> = (position: HoveredToken & C) => Subscribable<MaybeLoadingResult<(HoverAttachment & D) | null>> | PromiseLike<(HoverAttachment & D) | null>;
/**
 * Function that returns a Subscribable or PromiseLike of the ranges to be highlighted in the document.
 * If a Subscribable is returned, it may emit more than once to update the content, and must indicate when
 * it starts and stopped loading new content. It should emit a `null` result if the token has no highlights.
 *
 * @template C Extra context for the hovered token.
 */
export declare type DocumentHighlightProvider<C extends object> = (position: HoveredToken & C) => Subscribable<DocumentHighlight[]> | PromiseLike<DocumentHighlight[]>;
/**
 * @template C Extra context for the hovered token.
 * @template A The type of an action.
 */
export declare type ActionsProvider<C extends object, A> = (position: HoveredToken & C) => SubscribableOrPromise<A[] | null>;
/**
 * Function responsible for resolving the position of a hovered token
 * and its diff part to a full context including repository, commit ID and file path.
 *
 * @template C Extra context for the hovered token.
 */
export declare type ContextResolver<C extends object> = (hoveredToken: HoveredToken) => C;
/**
 * @template C Extra context for the hovered token.
 * @template D The type of the hover content data.
 * @template A The type of an action.
 */
export declare function createHoverifier<C extends object, D, A>({ closeButtonClicks, hoverOverlayElements, hoverOverlayRerenders, getHover, getDocumentHighlights, getActions, pinningEnabled, tokenize, selectionHighlightClassName, documentHighlightClassName, }: HoverifierOptions<C, D, A>): Hoverifier<C, D, A>;
//# sourceMappingURL=hoverifier.d.ts.map