import { Position, Range } from '@sourcegraph/extension-api-types';
import { LineOrPositionOrRange } from './types';
/**
 * A collection of methods needed to tell codeintellify how to look at the DOM. These are required for
 * ensuring that we don't rely on any sort of specific DOM structure.
 */
export interface DOMFunctions {
    /**
     * Get the element containing the code for a line from an event target.
     *
     * @param target is the event target.
     * @returns the element containing the code for a line or null if it can't be found. For example, the second <td> inside a <tr> on Sourcegraph and Github.
     */
    getCodeElementFromTarget: (target: HTMLElement) => HTMLElement | null;
    /**
     * Get the element containing the code for a line from a code view given a line number.
     *
     * @param codeView is the code view itself. For example, the <code> element on Sourcegraph or a <table> on GitHub.
     * @param part If the code view is a diff view, the part of the diff that the line number refers to.
     * @returns the element containing the code for the given line number or null if it can't be found.
     */
    getCodeElementFromLineNumber: (codeView: HTMLElement, line: number, part?: DiffPart) => HTMLElement | null;
    /**
     * Gets the line number for a given element containing code for a line.
     * When this function is called, it will be passed the result of either `getCodeElementFromTarget` or `getCodeElementFromLineNumber`.
     *
     * @param codeElement The element containing code for a line.
     * @returns The line number.
     */
    getLineNumberFromCodeElement: (codeElement: HTMLElement) => number;
    /**
     * If the code view is a diff view, must be provided to determine whether
     * a code element is from the base, head or unchanged part of the diff.
     * Must be `undefined` if the code view is not a diff view.
     *
     * @param codeElement is the element containing a line of code.
     * @returns The part of the diff `codeElement` belongs to
     */
    getDiffCodePart?: (codeElement: HTMLElement) => DiffPart;
    /**
     * Must return `true` if the first character in the code element is always
     * the diff indicator (`+`, `-` or space), `false` otherwise.
     *
     * @param codeElement is the element containing a line of code.
     */
    isFirstCharacterDiffIndicator?(codeElement: HTMLElement): boolean;
}
/**
 * Like `convertNode`, but idempotent.
 * The CSS class `annotated` is used to check if the cell is already converted.
 *
 * @param element The code `<td>` to convert.
 */
export declare function convertCodeElementIdempotent(element: HTMLElement): void;
/**
 * Helper function to determine whether the code view should be tokenized.
 */
export declare function shouldTokenize({ tokenize, overrideTokenize, }: {
    tokenize: boolean;
    overrideTokenize: boolean | undefined;
}): boolean;
/**
 * convertNode modifies a DOM node so that we can identify precisely token a user has clicked or hovered over.
 * On a code view, source code is typically wrapped in a HTML table cell. It may look like this:
 *
 * ```html
 * <td id="LC18" class="blob-code blob-code-inner js-file-line">
 *    <#textnode>\t</#textnode>
 *    <span class="pl-k">return</span>
 *    <#textnode>&amp;Router{namedRoutes: </#textnode>
 *    <span class="pl-c1">make</span>
 *    <#textnode>(</#textnode>
 *    <span class="pl-k">map</span>
 *    <#textnode>[</#textnode>
 *    <span class="pl-k">string</span>
 *    <#textnode>]*Route), KeepContext: </#textnode>
 *    <span class="pl-c1">false</span>
 *    <#textnode>}</#textnode>
 * </td>
 * ```
 *
 * The browser extension works by registering a hover event listeners on the <td> element. When the user hovers over
 * "return" (in the first <span> node) the event target will be the <span> node. We can use the event target to determine which line
 * and which character offset on that line to use to get tooltip data. But when the user hovers over "Router"
 * (in the second text node) the event target will be the <td> node, which lacks the appropriate specificity to request
 * tooltip data. To circumvent this, all we need to do is wrap every free text node in a <span> tag.
 *
 * In summary, convertNode effectively does this: https://gist.github.com/lebbe/6464236
 *
 * There are three additional edge cases we handle:
 *   1. some text nodes contain multiple discrete code tokens, like the second text node in the example above; by wrapping
 *     that text node in a <span> we lose the ability to distinguish whether the user is hovering over "Router" or "namedRoutes".
 *   2. there may be arbitrary levels of <span> nesting; in the example above, every <span> node has only one (text node) child, but
 *     in reality a <span> node could have multiple children, both text and element nodes
 *   3. on GitHub diff views (e.g. pull requests) the table cell contains an additional prefix character ("+" or "-" or " ", representing
 *     additions, deletions, and unchanged code, respectively); we want to make sure we don't count that character when computing the
 *     character offset for the line
 *   4. TODO(john) some code hosts transform source code before rendering; in the example above, the first text node may be a tab character
 *     or multiple spaces
 *
 * @param parentNode The node to convert.
 */
export declare function convertNode(parentNode: HTMLElement): void;
/**
 * Get the all of the text nodes under a given node in the DOM tree.
 *
 * @param node is the node in which you want to get all of the text nodes from it's children
 */
export declare const getTextNodes: (node: Node) => Node[];
/**
 * Returns the <span> (descendent of a <td> containing code) which contains text beginning
 * at the specified character offset, or within the specified offsets (1-indexed).
 * Will convert tokens in the code cell if needed.
 *
 * @param codeElement the element containing syntax highlighted code
 * @param offset object that specifies start character offset, and the optional end character offset (1-indexed)
 */
export declare function findElementWithOffset(codeElement: HTMLElement, { offsetStart, offsetEnd }: {
    offsetStart: number;
    offsetEnd?: number;
}, tokenize?: boolean): HTMLElement | undefined;
/**
 * Whether a line belongs to the base rev of the diff (removed), the head (added) or `null` if either (not changed).
 */
export declare type DiffPart = 'base' | 'head' | null;
/**
 * Returned when only the line is known.
 *
 * 1-indexed
 */
export interface Line {
    line: number;
}
export interface HoveredToken {
    /** 1-indexed */
    line: number;
    /** 1-indexed */
    character: number;
    part?: DiffPart;
}
/**
 * Determines the line and character offset for some source code, identified by its HTMLElement wrapper.
 * It works by traversing the DOM until the HTMLElement's TD ancestor. Once the ancestor is found, we traverse the DOM again
 * (this time the opposite direction) counting characters until the original target is found.
 * Returns undefined if line/char cannot be determined for the provided target.
 *
 * @param target The element to compute line & character offset for.
 * @param ignoreFirstChar Whether to ignore the first character on a line when computing character offset.
 */
export declare function locateTarget(target: HTMLElement, { getCodeElementFromTarget, getLineNumberFromCodeElement, getDiffCodePart, isFirstCharacterDiffIndicator, }: DOMFunctions): Line | HoveredToken | undefined;
export interface GetCodeElementsInRangeOptions extends Pick<DOMFunctions, 'getCodeElementFromLineNumber'> {
    codeView: HTMLElement;
    position?: LineOrPositionOrRange;
    part?: DiffPart;
}
export declare const getCodeElementsInRange: ({ codeView, position, part, getCodeElementFromLineNumber, }: GetCodeElementsInRangeOptions) => {
    /** 1-indexed line number */
    line: number;
    /** The element containing the code */
    element: HTMLElement;
}[];
/**
 * Returns the token `<span>` element in a code view for a given 1-indexed position or range.
 *
 * @param codeView The code view
 * @param positionOrRange 1-indexed position or range
 * @param domOptions Code-host specific implementations of DOM retrieval functions
 * @param part If the code view is a diff view, the part of the diff that the position refers to
 */
export declare const getTokenAtPositionOrRange: (codeView: HTMLElement, positionOrRange: Position | Range, { getCodeElementFromLineNumber, isFirstCharacterDiffIndicator, }: Pick<DOMFunctions, "isFirstCharacterDiffIndicator" | "getCodeElementFromLineNumber">, part?: "base" | "head" | null | undefined, tokenize?: boolean) => HTMLElement | undefined;
//# sourceMappingURL=token_position.d.ts.map